package turgen.output;

import java.util.Iterator;
import javax.swing.*;
import turgen.playlist.*;
import turgen.*;
import turgen.generator.SignalGenerator;
import turgen.plugin.*;
import turgen.config.*;



/**Output to the sound card*/
public class AudioOutput {
    
    
    static IntegerConfigEntry cfgInterRecordGap;
    static ConfigClass cfcAudioOutput;
    
    public static void init() {

        /*Initialize configuration class*/
        cfcAudioOutput = new ConfigClass("AudioOutput","Audio output");

        /*Initialize configuration entries*/
        cfgInterRecordGap = new IntegerConfigEntry("interRecordGap","Silence between files [ms]",500,0,5000);

        /*Add entries to the class*/
        cfcAudioOutput.addEntry(cfgInterRecordGap);

        /*Register configuration class*/
        Configuration.registerClass(cfcAudioOutput);
    }
    
    
    /**
     * 
     * @param items Playlist items
     * @param cmps GUI components to be disabled
     * @param repeat Repeat output in infinite loop
     * @param st Status object
     */
    public static synchronized void processPlaylistItems(Iterator<AbstractPlayListItem> items,JComponent[] cmps,TurgenStatus st,boolean repeat) {
        
        /*Disable components, clear status object*/
        ParallelGUIUpdater.setComponentsEnabled(cmps,false);    
        st.clearDirect();
        
        AbstractPlayListItem api;
        
        /*For each item*/
        while (items.hasNext()) {
            api = items.next();

            /*What is the plugin*/
            Plugin p = Turgen.pluginManager.getPlugin(api.pluginClass);

            /*If there is no plugin, report it and continue*/
            if (p==null) {
                ParallelGUIUpdater.showMessage("<HTML>Plugin \""+api.pluginClass+"\" <BR>specified in a playlist item not found.</HTML>");
                st.addFailedDirect();
                continue;
            }

            /*Convert an item and get SignalGenerator instructions*/
            int[] instr;
            try {
                instr = p.convertItem(api);
            }
            catch (Exception e) {
                ParallelGUIUpdater.showMessage(Utils.getTitledExceptionMessage("Conversion failed", e));
                st.addFailedDirect();
                e.printStackTrace();
                continue;
            }
            
            /*Execute generator*/
            try {
                SignalGenerator g = new SignalGenerator(api,instr,false,false,SignalGenerator.TYPE_AUDIO,repeat);
                g.run();
                /*Wait until it ends*/
                if (g.getCancelledType()==SignalGenerator.CANCEL_ALL) break;
                if (g.getErrorMessage()!=null) ParallelGUIUpdater.showMessage(g.getErrorMessage());
                if (items.hasNext()) Thread.sleep(cfgInterRecordGap.getAsInt());
            }
            catch(InterruptedException ie) {
                /*Someone pressed STOP or STOP ALL, that is OK*/
            }
            catch(Exception e) {
                /*Other execptions must be reported*/
                e.printStackTrace();
                ParallelGUIUpdater.showMessage(Utils.getTitledExceptionMessage("Audio output",e));
            }
            
        }

        /*Enable GUI components*/
        ParallelGUIUpdater.setComponentsEnabled(cmps,true);    

        
        
    }

}
