package turgen.output;

import java.util.Iterator;
import javax.swing.*;
import turgen.playlist.*;
import turgen.*;
import turgen.generator.*;
import turgen.plugin.*;
import turgen.config.*;

/**Output to a WAVE file*/
public class WaveOutput {

    public static ConfigClass cfcWaveOutput;
    public static BooleanConfigEntry cfgWOAutoRenameToWAV;
    
    public static void init() {

        /*Initialize configuration class*/
        cfcWaveOutput = new ConfigClass("WaveOutput","WAVE output");

        /*Initialize configuration entries*/
        cfgWOAutoRenameToWAV= new BooleanConfigEntry("autoRenameToWAV","Change extension to .wav",true);

        /*Add entries to the class*/
        cfcWaveOutput.addEntry(cfgWOAutoRenameToWAV);

        /*Register configuration class*/
        Configuration.registerClass(cfcWaveOutput);
        
    }

    /** Process playlist items of given indexes
     * 
     * @param items Playlist items
     * @param cmps Components to be re-enabled
     * @param st Status object
     * @param postProEnabled Indicates whether postprocessing is enabled
     */
    public static synchronized void processPlaylistItems(Iterator<AbstractPlayListItem> items,JComponent[] cmps,TurgenStatus st,boolean postProEnabled) {
        
        ParallelGUIUpdater.setComponentsEnabled(cmps, false);
        
        st.clearWaves();
        
        /*For each item*/
        while(items.hasNext()) {
            AbstractPlayListItem api = items.next();
            /*Determine plugin*/
            Plugin p = Turgen.pluginManager.getPlugin(api.pluginClass);
            /*If plugin not found, display a message and continue*/
            if (p==null) {
                ParallelGUIUpdater.showMessage("<HTML>Plugin \""+api.pluginClass+"\" <BR>specified in a playlist item not found.</HTML>");
                st.addFailedWaves();
                continue;
            }

            /*Convert an item and get SignalGenerator instructions*/
            int[] instr;
            try {
                instr = p.convertItem(api);
            }
            catch (Exception e) {
                ParallelGUIUpdater.showMessage(Utils.getTitledExceptionMessage("Conversion failed", e));
                st.addFailedWaves();
                e.printStackTrace();
                continue;
            }
            
            /*Run the signal generator*/
            try {
                SignalGenerator g = new SignalGenerator(api,instr,cfgWOAutoRenameToWAV.getAsBoolean(),postProEnabled,SignalGenerator.TYPE_WAVE,false);
                
                /*Each generator has its own thread*/
                Thread t = new Thread((Runnable)g);
                t.start();
                
            }
            catch(Exception e) {
                /*Exceptions are of no good*/
                e.printStackTrace();
                ParallelGUIUpdater.showMessage(Utils.getTitledExceptionMessage("WAVE output",e));
            }
            
        }
        
        ParallelGUIUpdater.setComponentsEnabled(cmps, true);
        
    }

}
